@@ -10,10 +10,14 @@
 #include <io.h>
 
 #ifndef LIBXML_DLL_IMPORT
+#if !defined(STATIC)
+#define LIBXML_DLL_IMPORT __declspec(dllimport)
+#else
 #define LIBXML_DLL_IMPORT
 #endif
-#define SOCKLEN_T int
+#endif
 
+#define SOCKLEN_T int
 #ifdef NEED_SOCKETS
 #include <winsock2.h>
 
@@ -99,7 +103,7 @@ static int isnan (double d) {
 /* Microsoft's C runtime names all non-ANSI functions with a leading
    underscore. Since functionality is still the same, they can be used. */
 #ifdef _MSC_VER
-#include <libxml/xmlversion.h>
+#include <libxml/xmlwin32version.h>
 #ifndef WITH_TRIO
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
