@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@w3.org
+ * daniel@veillard.com
  */
 
 #include "libxml.h"
@@ -12,7 +12,6 @@
 #include <stdio.h>
 #include <stdarg.h>
 
-#include <libxml/xmlversion.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/uri.h>
 #include <libxml/catalog.h>
@@ -26,6 +25,7 @@ int main(int argc, char **argv) {
 
     xmlCatalogDump(stdout);
     xmlCatalogCleanup();
+    xmlCleanupParser();
     xmlMemoryDump();
 #endif
     return(0);
